(* $Id$ *)


(*
   Numerical domain for constant propagation analysis.
 *)


(*
   An abstract element is either Bot, or (Val n) where n is an integer, or Top.
   The concretization function γ is defined as follows:

                  Abstract element                Concretization
                         Bot                            ∅
                        Val n                          {n}
                         Top                            ℤ

   The abstraction function α is defined by α(∅) = Bot, α({n}) = n for every
   integer n, and α(S) = Top for every subset S ⊆ ℤ of cardinal at least two.
 *)
type t =
  | Bot
  | Val of Z.t
  | Top

open Z
open Z.Compare

let print fmt =
  function
  | Bot -> Format.pp_print_string fmt "⊥"
  | Val x -> pp_print fmt x
  | Top -> Format.pp_print_string fmt "⊤"

let bot = Bot
let top = Top

let equal a b =
  match (a, b) with
  | (Bot, Bot)
  | (Top, Top) -> true
  | (Val x, Val y) -> x = y
  | ((Bot | Val _ | Top), (Bot | Val _ | Top)) -> false

let leq a b =
  match (a, b) with
  | (Bot, _)
  | (_, Top) -> true
  | (_, Bot)
  | (Top, _) -> false
  | (Val x, Val y) -> x = y

let glb a b =
  failwith "DomConstant.glb: not implemented. Go ahead!"

let lub a b =
  failwith "DomConstant.lub: not implemented. Go ahead!"

let abs x = Val (of_int x)

let empty a = equal a bot

module Op =
struct
  let add a b =
    failwith "DomConstant.add: not implemented. Go ahead!"

  let sub a b =
    failwith "DomConstant.sub: not implemented. Go ahead!"

  let mul a b =
    failwith "DomConstant.mul: not implemented. Go ahead!"

  let div a b =
    failwith "DomConstant.div: not implemented. Go ahead!"

  let equality a b =
    failwith "DomConstant.equality: not implemented. Go ahead!"

  let inequality a b =
    failwith "DomConstant.inequality: not implemented. Go ahead!"
end
