(* $Id$ *)


(*
   Numerical domain for sign analysis.
 *)


(*
   An abstract element is a triple of booleans (n, z, p) where n stands for
   negative, z for zero and p for positive.  The concretization of (n, z, p) is
   the set of integers x satisfying the following formula:

                  (n ∧ x < 0)  ∨  (z ∧ x = 0)  ∨  (p ∧ x > 0)

   Examples:         Abstract triple            Concretization
                  (true , false, true )         { x | x ≠ 0 }
                  (false, true , false)         { 0 }
                  (false, true , true )         { x | x ≥ 0 }
                  (true , false, false)         { x | x < 0 }

   Accordingly, the abstraction function α is defined by

                α(S)  =  (∃x ∈ S : x < 0, 0 ∈ S, ∃x ∈ S : x > 0)

   for every subset S ⊆ ℤ.

   Put differently, the sign abstract domain is induced by the partition of the
   integers into negative, zero and positive integers.  Triples (n, z, p) encode
   unions of classes in the obvious way.
 *)
type t = bool * bool * bool

let print fmt =
  function
  | (false, false, false) -> Format.pp_print_string fmt "⊥"
  | (true , false, false) -> Format.pp_print_string fmt "-"
  | (false, true , false) -> Format.pp_print_string fmt "0"
  | (false, false, true ) -> Format.pp_print_string fmt "+"
  | (true , true , false) -> Format.pp_print_string fmt "-0"
  | (true , false, true ) -> Format.pp_print_string fmt "-+"
  | (false, true , true ) -> Format.pp_print_string fmt "0+"
  | (true , true , true ) -> Format.pp_print_string fmt "⊤"

let bot = (false, false, false)
let top = (true , true , true )

let equal = (=)

let leq (n1, z1, p1) (n2, z2, p2) =
  failwith "DomSign.leq: not implemented. Go ahead!"

let glb (n1, z1, p1) (n2, z2, p2) =
  failwith "DomSign.glb: not implemented. Go ahead!"

let lub (n1, z1, p1) (n2, z2, p2) =
  failwith "DomSign.lub: not implemented. Go ahead!"

let abs x =
  failwith "DomSign.abs: not implemented. Go ahead!"

let empty a =
  failwith "DomSign.empty: not implemented. Go ahead!"

module Op =
struct
  let neg (n, _, _) = n
  let zer (_, z, _) = z
  let pos (_, _, p) = p

  let add a b =
    failwith "DomSign.add: not implemented. Go ahead!"

  let sub a b =
    failwith "DomSign.sub: not implemented. Go ahead!"

  let mul a b =
    failwith "DomSign.mul: not implemented. Go ahead!"

  let div a b =
    failwith "DomSign.div: not implemented. Go ahead!"

  let equality a b =
    failwith "DomSign.equality: not implemented. Go ahead!"

  let inequality a b =
    failwith "DomSign.inequality: not implemented. Go ahead!"
end
